@@ -2,7 +2,6 @@ package iotdataplane_test
 
 import (
 	"fmt"
-	"github.com/stretchr/testify/assert"
 	"testing"
 
 	"github.com/aws/aws-sdk-go/aws"
@@ -18,14 +17,19 @@ func TestRequireEndpointIfRegionProvided(t *testing.T) {
 	req, _ := svc.GetThingShadowRequest(nil)
 	err := req.Build()
 
-	assert.Equal(t, "", svc.Endpoint)
-	assert.Error(t, err)
-	assert.Equal(t, aws.ErrMissingEndpoint, err)
+	if e, a := "", svc.Endpoint; e != a {
+		t.Errorf("expect %v, got %v", e, a)
+	}
+	if err == nil {
+		t.Errorf("expect error, got none")
+	}
+	if e, a := aws.ErrMissingEndpoint, err; e != a {
+		t.Errorf("expect %v, got %v", e, a)
+	}
 }
 
 func TestRequireEndpointIfNoRegionProvided(t *testing.T) {
 	svc := iotdataplane.New(unit.Session, &aws.Config{
-		Region:                 aws.String(""),
 		DisableParamValidation: aws.Bool(true),
 	})
 	fmt.Println(svc.ClientInfo.SigningRegion)
@@ -33,9 +37,15 @@ func TestRequireEndpointIfNoRegionProvided(t *testing.T) {
 	req, _ := svc.GetThingShadowRequest(nil)
 	err := req.Build()
 
-	assert.Equal(t, "", svc.Endpoint)
-	assert.Error(t, err)
-	assert.Equal(t, aws.ErrMissingEndpoint, err)
+	if e, a := "", svc.Endpoint; e != a {
+		t.Errorf("expect %v, got %v", e, a)
+	}
+	if err == nil {
+		t.Errorf("expect error, got none")
+	}
+	if e, a := aws.ErrMissingEndpoint, err; e != a {
+		t.Errorf("expect %v, got %v", e, a)
+	}
 }
 
 func TestRequireEndpointUsed(t *testing.T) {
@@ -47,6 +57,10 @@ func TestRequireEndpointUsed(t *testing.T) {
 	req, _ := svc.GetThingShadowRequest(nil)
 	err := req.Build()
 
-	assert.Equal(t, "https://endpoint", svc.Endpoint)
-	assert.NoError(t, err)
+	if e, a := "https://endpoint", svc.Endpoint; e != a {
+		t.Errorf("expect %v, got %v", e, a)
+	}
+	if err != nil {
+		t.Errorf("expect no error, got %v", err)
+	}
 }
